@@ -81,6 +81,8 @@ static char *darwin_error_str (int result) {
     return "transaction aborted";
   case kIOReturnNotResponding:
     return "device not responding";
+  case kIOReturnOverrun:
+    return "data overrun";
   default:
     return "unknown error";
   }
@@ -150,7 +152,8 @@ static usb_device_t **usb_get_next_device (io_iterator_t deviceIterator, UInt32
   io_cf_plugin_ref_t *plugInInterface = NULL;
   usb_device_t **device;
   io_service_t usbDevice;
-  long result, score;
+  long result;
+  SInt32 score;
 
   if (!IOIteratorIsValid (deviceIterator) || !(usbDevice = IOIteratorNext(deviceIterator)))
     return NULL;
@@ -652,6 +655,9 @@ static void darwin_close (struct libusb_device_handle *dev_handle) {
   usbi_remove_pollfd (HANDLE_CTX (dev_handle), priv->fds[0]);
   close (priv->fds[1]);
   close (priv->fds[0]);
+
+  dpriv->device = NULL;
+  priv->fds[0] = priv->fds[1] = -1;
 }
 
 static int darwin_get_configuration(struct libusb_device_handle *dev_handle, int *config) {
@@ -767,7 +773,7 @@ static int darwin_claim_interface(struct libusb_device_handle *dev_handle, int i
   io_service_t          usbInterface = IO_OBJECT_NULL;
   IOReturn kresult;
   IOCFPlugInInterface **plugInInterface = NULL;
-  long                  score;
+  SInt32                score;
   uint8_t               new_config;
 
   /* current interface */
@@ -1296,12 +1302,17 @@ static void darwin_bulk_callback (struct usbi_transfer *itransfer, kern_return_t
     usbi_handle_transfer_cancellation(itransfer);
     return;
   case kIOUSBPipeStalled:
-    _usbi_log (ITRANSFER_CTX (itransfer), LOG_LEVEL_WARNING, "unsupported control request");
+    _usbi_log (ITRANSFER_CTX (itransfer), LOG_LEVEL_WARNING, "bulk error. pipe is stalled");
     status = LIBUSB_TRANSFER_STALL;
     
+    break;
+  case kIOReturnOverrun:
+    _usbi_log (ITRANSFER_CTX (itransfer), LOG_LEVEL_ERROR, "bulk error. data overrun", darwin_error_str (result));
+    status = LIBUSB_TRANSFER_OVERFLOW;
+
     break;
   default:
-    _usbi_log (ITRANSFER_CTX (itransfer), LOG_LEVEL_ERROR, "control error = %s", darwin_error_str (result));
+    _usbi_log (ITRANSFER_CTX (itransfer), LOG_LEVEL_ERROR, "bulk error = %s (value = 0x%08x)", darwin_error_str (result), result);
     status = LIBUSB_TRANSFER_ERROR;
   }
 
@@ -1336,6 +1347,11 @@ static void darwin_isoc_callback (struct usbi_transfer *itransfer, kern_return_t
     _usbi_log (ITRANSFER_CTX (itransfer), LOG_LEVEL_WARNING, "unsupported control request");
     status = LIBUSB_TRANSFER_STALL;
 
+    break;
+  case kIOReturnOverrun:
+    _usbi_log (ITRANSFER_CTX (itransfer), LOG_LEVEL_ERROR, "bulk error. data overrun", darwin_error_str (result));
+    status = LIBUSB_TRANSFER_OVERFLOW;
+
     break;
   default:
     _usbi_log (ITRANSFER_CTX (itransfer), LOG_LEVEL_ERROR, "control error = %s", darwin_error_str (result));
